import express from 'express'
import jwt from 'jwt-simple'
import mongoose from 'mongoose'
import bcrypt from 'bcrypt'
import { writeFile } from 'fs'

import User from '../models/user'
import config from '../config/database'

const authenticate = express.Router()

mongoose.connect(config.database, {
  connectTimeoutMS: 1000
})

function initDB() {
  User.findOne({username: "admin"}, (err, user) => {
    if (err) {
      console.log(err.toString())
    }
    if (!user) {
      console.log("Uninitialized database. Creating admin user.")
      const password = Math.random().toString(36).substr(2)
      const admin = new User({
        username: "admin",
        name: "Administrator",
        password
      })
      admin.save()
      writeFile("/admin_password", password, (err) => {
        if (err) {
          console.log(err.toString())
        }
        console.log("Wrote password in text file.")
      })
    }
  })
}

initDB()

authenticate.post('/login', (req, res) => {
  console.log("Trying to log in.")
  const { username, password } = req.body
  if (!username || !password) {
    return res.status(400).json({ 
      success: false, 
      msg: 'Parameters name and password are required' 
    })
  }

  User.findOne({ username }, (err, user) => {
    console.log('hoppla!')
    if (err) {
      return res.status(400).json({ 
        success: false, 
        msg: err.toString() 
      })
    }
    if (!user) {
      return res.status(400).json({ 
        success: false, 
        msg: 'Authentication failed. User not found.' 
      })
    }
    console.log(password, user)
    if (!bcrypt.compareSync(password, user.password)) {
      return res.status(400).json({ 
        success: false, 
        msg: 'Authentication failed. Wrong password' 
      })
    } else {
      const token = jwt.encode({
        exp: Math.floor(Date.now() / 1000) + 24*60*60,
        data: user
      }, config.secret)
      return res.json({ 
        success: true, 
        token 
      })
    }
  })
})

const verify = express.Router()

verify.use( (req, res, next) => {
  console.log("Trying to authenticate token.")
  const token = req.headers['x-access-token']

  if (token) {

    const decoded = jwt.decode(token, config.secret)
    req.decoded = decoded
    next()

  } else {
    return res.status(403).send({
      success: false,
      message: 'No token provided.'
    })
  }
})

export { authenticate, verify }